@@ -10,7 +10,9 @@ import copy
 import json
 import re
 import time
+from collections.abc import Generator
 from pathlib import Path, PurePosixPath
+from typing import Any
 from urllib.parse import (
     parse_qs,
     parse_qsl,
@@ -57,15 +59,15 @@ class Zotero:
 
     def __init__(
         self,
-        library_id=None,
-        library_type=None,
-        api_key=None,
-        preserve_json_order=False,
-        locale="en-US",
-        local=False,
-        client=None,
-    ):
-        self.client = None
+        library_id: str | int | None = None,
+        library_type: str | None = None,
+        api_key: str | None = None,
+        preserve_json_order: bool = False,
+        locale: str = "en-US",
+        local: bool = False,
+        client: httpx.Client | None = None,
+    ) -> None:
+        self.client: httpx.Client | None = None
         """Store Zotero credentials"""
         if not local:
             self.endpoint = "https://api.zotero.org"
@@ -92,9 +94,9 @@ class Zotero:
                 stacklevel=2,
             )
         self.locale = locale
-        self.url_params = None
+        self.url_params: dict[str, Any] | None = None
         self.tag_data = False
-        self.request = None
+        self.request: httpx.Response | None = None
         self.snapshot = False
         self.client = client or httpx.Client(
             headers=self.default_headers(),
@@ -106,7 +108,7 @@ class Zotero:
         self.fmt = re.compile(r"(?<=format=)\w+")
         self.content = re.compile(r"(?<=content=)\w+")
         # JSON by default
-        self.formats = {
+        self.formats: dict[str, str] = {
             "application/atom+xml": "atom",
             "application/x-bibtex": "bibtex",
             "application/json": "json",
@@ -135,7 +137,7 @@ class Zotero:
             "application/postscript": "postscript",
             "application/rtf": "rtf",
         }
-        self.processors = {
+        self.processors: dict[str, Any] = {
             "bib": self._bib_processor,
             "citation": self._citation_processor,
             "bibtex": self._bib_processor,
@@ -153,29 +155,29 @@ class Zotero:
             "json": self._json_processor,
             "html": self._bib_processor,
         }
-        self.links = None
-        self.self_link = {}
-        self.templates = {}
-        self.savedsearch = None
+        self.links: dict[str, str] | None = None
+        self.self_link: dict[str, Any] | httpx.URL = {}
+        self.templates: dict[str, Any] = {}
+        self.savedsearch: Any = None
         # backoff handling: timestamp when backoff expires (0.0 = no backoff)
-        self.backoff_until = 0.0
+        self.backoff_until: float = 0.0
 
-    def __del__(self):
+    def __del__(self) -> None:
         """Remove client before cleanup."""
         # this isn't guaranteed to run, but that's OK
         if c := self.client:
             c.close()
 
     @property
-    def __version__(self):
+    def __version__(self) -> str:
         """Return the version of the pyzotero library."""
         return pz.__version__
 
-    def _check_for_component(self, url, component):
+    def _check_for_component(self, url: str | None, component: str) -> bool:
         """Check a url path query fragment for a specific query parameter."""
         return bool(parse_qs(url).get(component))
 
-    def _striplocal(self, url):
+    def _striplocal(self, url: str) -> str:
         """Remove the leading '/api' substring from urls if running in local mode."""
         if self.local:
             parsed = urlparse(url)
@@ -185,17 +187,17 @@ class Zotero:
             return urlunparse(replaced)
         return url
 
-    def _set_backoff(self, duration):
+    def _set_backoff(self, duration: str | float) -> None:
         """Set backoff expiration time."""
         self.backoff_until = time.time() + float(duration)
 
-    def _check_backoff(self):
+    def _check_backoff(self) -> None:
         """Wait if backoff is active."""
         remainder = self.backoff_until - time.time()
         if remainder > 0.0:
             time.sleep(remainder)
 
-    def default_headers(self):
+    def default_headers(self) -> dict[str, str]:
         """Return headers that are always OK to include."""
         _headers = {
             "User-Agent": f"Pyzotero/{pz.__version__}",
@@ -205,7 +207,7 @@ class Zotero:
             _headers["Authorization"] = f"Bearer {self.api_key}"
         return _headers
 
-    def _cache(self, response, key):
+    def _cache(self, response: httpx.Response, key: str) -> Any:
         """Add a retrieved template to the cache for 304 checking.
 
         Accepts a dict and key name, adds the retrieval time, and adds both
@@ -220,7 +222,9 @@ class Zotero:
         return copy.deepcopy(response.json())
 
     @cleanwrap
-    def _cleanup(self, to_clean, allow=()):
+    def _cleanup(
+        self, to_clean: dict[str, Any], allow: tuple[str, ...] = ()
+    ) -> dict[str, Any]:
         """Remove keys we added for internal use."""
         # this item's been retrieved from the API, we only need the 'data' entry
         if to_clean.keys() == ["links", "library", "version", "meta", "key", "data"]:
@@ -229,7 +233,9 @@ class Zotero:
             k: v for k, v in to_clean.items() if k in allow or k not in self.temp_keys
         }
 
-    def _retrieve_data(self, request: str | None = None, params=None):
+    def _retrieve_data(
+        self, request: str | None = None, params: dict[str, Any] | None = None
+    ) -> httpx.Response:
         """Retrieve Zotero items via the API.
 
         Combine endpoint and request to access the specific resource.
@@ -294,11 +300,13 @@ class Zotero:
             self._set_backoff(backoff)
         return self.request
 
-    def _extract_links(self):
+    def _extract_links(self) -> dict[str, str] | None:
         """Extract self, first, next, last links from a request response."""
-        extracted = {}
+        extracted: dict[str, str] = {}
         try:
             for key, value in self.request.links.items():
+                if key is None:
+                    continue
                 parsed = urlparse(value["url"])
                 fragment = urlunparse(("", "", parsed.path, "", parsed.query, ""))
                 extracted[key] = fragment
@@ -316,7 +324,9 @@ class Zotero:
         else:
             return extracted
 
-    def _updated(self, url, payload, template=None):
+    def _updated(
+        self, url: str, payload: dict[str, Any], template: str | None = None
+    ) -> bool:
         """Call to see if a template request returns 304.
 
         Accepts:
@@ -328,6 +338,8 @@ class Zotero:
         assumed to be fresh, and will immediately return False if found.
         """
         # If the template is more than an hour old, try a 304
+        if template is None:
+            return False
         if (
             abs(
                 whenever.ZonedDateTime.now("Europe/London").py_datetime()
@@ -358,7 +370,7 @@ class Zotero:
         # Still plenty of life left in't
         return False
 
-    def add_parameters(self, **params):
+    def add_parameters(self, **params: Any) -> None:
         """Add URL parameters.
 
         Also ensure that only valid format/content combinations are requested.
@@ -391,7 +403,7 @@ class Zotero:
         # Merge preserved params with new params (new params override existing ones)
         self.url_params = {**preserved_params, **params}
 
-    def _build_query(self, query_string, no_params=False):
+    def _build_query(self, query_string: str, no_params: bool = False) -> str:
         """Set request parameters.
 
         Will always add the user ID if it hasn't been specifically set by an API method.
@@ -407,7 +419,7 @@ class Zotero:
         return query
 
     @retrieve
-    def publications(self):
+    def publications(self) -> str:
         """Return the contents of My Publications."""
         if self.library_type != "users":
             msg = "This API call does not exist for group libraries"
@@ -416,22 +428,22 @@ class Zotero:
         return self._build_query(query_string)
 
     # The following methods are Zotero Read API calls
-    def num_items(self):
+    def num_items(self) -> int:
         """Return the total number of top-level items in the library."""
         query = "/{t}/{u}/items/top"
         return self._totals(query)
 
-    def count_items(self):
+    def count_items(self) -> int:
         """Return the count of all items in a group / library."""
         query = "/{t}/{u}/items"
         return self._totals(query)
 
-    def num_collectionitems(self, collection):
+    def num_collectionitems(self, collection: str) -> int:
         """Return the total number of items in the specified collection."""
         query = f"/{self.library_type}/{self.library_id}/collections/{collection.upper()}/items"
         return self._totals(query)
 
-    def _totals(self, query):
+    def _totals(self, query: str) -> int:
         """General method for returning total counts."""
         self.add_parameters(limit=1)
         query = self._build_query(query)
@@ -441,25 +453,25 @@ class Zotero:
         return int(self.request.headers["Total-Results"])
 
     @retrieve
-    def key_info(self, **kwargs):
+    def key_info(self, **kwargs: Any) -> str:
         """Retrieve info about the permissions associated with the key."""
         query_string = f"/keys/{self.api_key}"
         return self._build_query(query_string)
 
     @retrieve
-    def items(self, **kwargs):
+    def items(self, **kwargs: Any) -> str:
         """Get user items."""
         query_string = "/{t}/{u}/items"
         return self._build_query(query_string)
 
     @retrieve
-    def settings(self, **kwargs):
+    def settings(self, **kwargs: Any) -> str:
         """Get synced user settings."""
         query_string = "/{t}/{u}/settings"
         return self._build_query(query_string)
 
     @retrieve
-    def fulltext_item(self, itemkey, **kwargs):
+    def fulltext_item(self, itemkey: str, **kwargs: Any) -> str:
         """Get full-text content for an item."""
         query_string = (
             f"/{self.library_type}/{self.library_id}/items/{itemkey}/fulltext"
@@ -467,7 +479,7 @@ class Zotero:
         return self._build_query(query_string)
 
     @backoff_check
-    def set_fulltext(self, itemkey, payload):
+    def set_fulltext(self, itemkey: str, payload: dict[str, Any]) -> httpx.Response:
         """Set full-text data for an item.
 
         <itemkey> should correspond to an existing attachment item.
@@ -486,11 +498,11 @@ class Zotero:
             json=payload,
         )
 
-    def new_fulltext(self, since):
+    def new_fulltext(self, since: int) -> Any:
         """Retrieve list of full-text content items and versions newer than <since>."""
         query_string = f"/{self.library_type}/{self.library_id}/fulltext"
-        headers = {}
-        params = {"since": since}
+        headers: dict[str, str] = {}
+        params: dict[str, Any] = {"since": since}
         self._check_backoff()
         resp = self.client.get(
             build_url(self.endpoint, query_string),
@@ -506,7 +518,7 @@ class Zotero:
             self._set_backoff(backoff)
         return resp.json()
 
-    def item_versions(self, **kwargs):
+    def item_versions(self, **kwargs: Any) -> Any:
         """Return dict associating item keys to versions.
 
         Accepts a since= parameter in kwargs to limit the data to those updated since.
@@ -516,7 +528,7 @@ class Zotero:
         kwargs["format"] = "versions"
         return self.items(**kwargs)
 
-    def collection_versions(self, **kwargs):
+    def collection_versions(self, **kwargs: Any) -> Any:
         """Return dict associating collection keys to versions.
 
         Accepts a since= parameter in kwargs to limit the data to those updated since.
@@ -526,7 +538,7 @@ class Zotero:
         kwargs["format"] = "versions"
         return self.collections(**kwargs)
 
-    def last_modified_version(self, **kwargs):
+    def last_modified_version(self, **kwargs: Any) -> int:
         """Get the last modified user or group library version."""
         # This MUST be a multiple-object request, limit param notwithstanding
         self.items(limit=1)
@@ -534,25 +546,25 @@ class Zotero:
         return int(lmv)
 
     @retrieve
-    def top(self, **kwargs):
+    def top(self, **kwargs: Any) -> str:
         """Get user top-level items."""
         query_string = "/{t}/{u}/items/top"
         return self._build_query(query_string)
 
     @retrieve
-    def trash(self, **kwargs):
+    def trash(self, **kwargs: Any) -> str:
         """Get all items in the trash."""
         query_string = "/{t}/{u}/items/trash"
         return self._build_query(query_string)
 
     @retrieve
-    def searches(self, **kwargs):
+    def searches(self, **kwargs: Any) -> str:
         """Get saved searches."""
         query_string = "/{t}/{u}/searches"
         return self._build_query(query_string)
 
     @retrieve
-    def deleted(self, **kwargs):
+    def deleted(self, **kwargs: Any) -> str:
         """Get all deleted items (requires since= parameter)."""
         if "limit" not in kwargs:
             # Currently deleted API doesn't respect limit leaving it out by
@@ -562,20 +574,22 @@ class Zotero:
         return self._build_query(query_string)
 
     @retrieve
-    def item(self, item, **kwargs):
+    def item(self, item: str, **kwargs: Any) -> str:
         """Get a specific item."""
         query_string = f"/{self.library_type}/{self.library_id}/items/{item.upper()}"
         return self._build_query(query_string)
 
     @retrieve
-    def file(self, item, **kwargs):
+    def file(self, item: str, **kwargs: Any) -> str:
         """Get the file from a specific item."""
         query_string = (
             f"/{self.library_type}/{self.library_id}/items/{item.upper()}/file"
         )
         return self._build_query(query_string, no_params=True)
 
-    def dump(self, itemkey, filename=None, path=None):
+    def dump(
+        self, itemkey: str, filename: str | None = None, path: str | None = None
+    ) -> None:
         """Dump a file attachment to disk, with optional filename and path."""
         if not filename:
             filename = self.item(itemkey)["data"]["filename"]
@@ -588,7 +602,7 @@ class Zotero:
             f.write(file)
 
     @retrieve
-    def children(self, item, **kwargs):
+    def children(self, item: str, **kwargs: Any) -> str:
         """Get a specific item's child items."""
         query_string = (
             f"/{self.library_type}/{self.library_id}/items/{item.upper()}/children"
@@ -596,25 +610,25 @@ class Zotero:
         return self._build_query(query_string)
 
     @retrieve
-    def collection_items(self, collection, **kwargs):
+    def collection_items(self, collection: str, **kwargs: Any) -> str:
         """Get a specific collection's items."""
         query_string = f"/{self.library_type}/{self.library_id}/collections/{collection.upper()}/items"
         return self._build_query(query_string)
 
     @retrieve
-    def collection_items_top(self, collection, **kwargs):
+    def collection_items_top(self, collection: str, **kwargs: Any) -> str:
         """Get a specific collection's top-level items."""
         query_string = f"/{self.library_type}/{self.library_id}/collections/{collection.upper()}/items/top"
         return self._build_query(query_string)
 
     @retrieve
-    def collection_tags(self, collection, **kwargs):
+    def collection_tags(self, collection: str, **kwargs: Any) -> str:
         """Get a specific collection's tags."""
         query_string = f"/{self.library_type}/{self.library_id}/collections/{collection.upper()}/tags"
         return self._build_query(query_string)
 
     @retrieve
-    def collection(self, collection, **kwargs):
+    def collection(self, collection: str, **kwargs: Any) -> str:
         """Get user collection."""
         query_string = (
             f"/{self.library_type}/{self.library_id}/collections/{collection.upper()}"
@@ -622,20 +636,20 @@ class Zotero:
         return self._build_query(query_string)
 
     @retrieve
-    def collections(self, **kwargs):
+    def collections(self, **kwargs: Any) -> str:
         """Get user collections."""
         query_string = "/{t}/{u}/collections"
         return self._build_query(query_string)
 
-    def all_collections(self, collid=None):
+    def all_collections(self, collid: str | None = None) -> list[Any]:
         """Retrieve all collections and subcollections.
 
         Works for top-level collections or for a specific collection.
         Works at all collection depths.
         """
-        all_collections = []
+        all_collections: list[Any] = []
 
-        def subcoll(clct):
+        def subcoll(clct: Any) -> None:
             """Recursively add collections to a flat master list."""
             all_collections.append(clct)
             if clct["meta"].get("numCollections", 0) > 0:
@@ -653,32 +667,32 @@ class Zotero:
         return all_collections
 
     @retrieve
-    def collections_top(self, **kwargs):
+    def collections_top(self, **kwargs: Any) -> str:
         """Get top-level user collections."""
         query_string = "/{t}/{u}/collections/top"
         return self._build_query(query_string)
 
     @retrieve
-    def collections_sub(self, collection, **kwargs):
+    def collections_sub(self, collection: str, **kwargs: Any) -> str:
         """Get subcollections for a specific collection."""
         query_string = f"/{self.library_type}/{self.library_id}/collections/{collection.upper()}/collections"
         return self._build_query(query_string)
 
     @retrieve
-    def groups(self, **kwargs):
+    def groups(self, **kwargs: Any) -> str:
         """Get user groups."""
         query_string = "/users/{u}/groups"
         return self._build_query(query_string)
 
     @retrieve
-    def tags(self, **kwargs):
+    def tags(self, **kwargs: Any) -> str:
         """Get tags."""
         query_string = "/{t}/{u}/tags"
         self.tag_data = True
         return self._build_query(query_string)
 
     @retrieve
-    def item_tags(self, item, **kwargs):
+    def item_tags(self, item: str, **kwargs: Any) -> str:
         """Get tags for a specific item."""
         query_string = (
             f"/{self.library_type}/{self.library_id}/items/{item.upper()}/tags"
@@ -686,18 +700,18 @@ class Zotero:
         self.tag_data = True
         return self._build_query(query_string)
 
-    def all_top(self, **kwargs):
+    def all_top(self, **kwargs: Any) -> Any:
         """Retrieve all top-level items."""
         return self.everything(self.top(**kwargs))
 
-    @retrieve
-    def follow(self):
+    @retrieve  # ty: ignore[invalid-argument-type]
+    def follow(self) -> str | None:
         """Return the result of the call to the URL in the 'Next' link."""
         if n := self.links.get("next"):
             return self._striplocal(n)
         return None
 
-    def iterfollow(self):
+    def iterfollow(self) -> Generator[Any, None, None]:
         """Return generator for self.follow()."""
         # use same criterion as self.follow()
         while True:
@@ -706,7 +720,7 @@ class Zotero:
             else:
                 return
 
-    def makeiter(self, func):
+    def makeiter(self, func: Any) -> Generator[Any, None, None]:
         """Return a generator of func's results."""
         if self.links is None or "self" not in self.links:
             msg = "makeiter() requires a previous API call with pagination links"
@@ -715,7 +729,7 @@ class Zotero:
         self.links["next"] = self.links["self"]
         return self.iterfollow()
 
-    def everything(self, query):
+    def everything(self, query: Any) -> Any:
         """Retrieve all items in the library for a particular query.
 
         This method will override the 'limit' parameter if it's been set.
@@ -732,7 +746,7 @@ class Zotero:
                 items.entries.extend(self.follow().entries)
         return items
 
-    def get_subset(self, subset):
+    def get_subset(self, subset: list[str]) -> list[Any]:
         """Retrieve a subset of items.
 
         Accepts a single argument: a list of item IDs.
@@ -742,7 +756,7 @@ class Zotero:
             raise ze.TooManyItemsError(err)
         # remember any url parameters that have been set
         params = self.url_params
-        retr = []
+        retr: list[Any] = []
         for itm in subset:
             retr.append(self.item(itm))
             self.url_params = params
@@ -751,7 +765,7 @@ class Zotero:
         return retr
 
     # The following methods process data returned by Read API calls
-    def _json_processor(self, retrieved):
+    def _json_processor(self, retrieved: Any) -> list[Any]:
         """Format and return data from API calls which return Items."""
         # send entries to _tags_data if there's no JSON
         try:
@@ -760,7 +774,7 @@ class Zotero:
             return self._tags_data(retrieved)
         return items
 
-    def _csljson_processor(self, retrieved):
+    def _csljson_processor(self, retrieved: Any) -> list[Any]:
         """Return a list of dicts which are dumped CSL JSON."""
         items = [
             json.loads(entry["content"][0]["value"]) for entry in retrieved.entries
@@ -768,29 +782,29 @@ class Zotero:
         self.url_params = None
         return items
 
-    def _bib_processor(self, retrieved):
+    def _bib_processor(self, retrieved: Any) -> list[str]:
         """Return a list of strings formatted as HTML bibliography entries."""
         items = [bib["content"][0]["value"] for bib in retrieved.entries]
         self.url_params = None
         return items
 
-    def _citation_processor(self, retrieved):
+    def _citation_processor(self, retrieved: Any) -> list[str]:
         """Return a list of strings formatted as HTML citation entries."""
         items = [cit["content"][0]["value"] for cit in retrieved.entries]
         self.url_params = None
         return items
 
-    def _tags_data(self, retrieved):
+    def _tags_data(self, retrieved: Any) -> list[str]:
         """Format and return data from API calls which return Tags."""
         self.url_params = None
         return [t["tag"] for t in retrieved]
 
     # The following methods are Write API calls
-    def item_template(self, itemtype, linkmode=None):
+    def item_template(self, itemtype: str, linkmode: str | None = None) -> Any:
         """Get a template for a new item."""
         # if we have a template and it hasn't been updated since we stored it
         template_name = f"item_template_{itemtype}_{linkmode or ''}"
-        params = {"itemType": itemtype}
+        params: dict[str, Any] = {"itemType": itemtype}
         # Set linkMode parameter for API request if itemtype is attachment
         if itemtype == "attachment":
             params["linkMode"] = linkmode
@@ -806,14 +820,16 @@ class Zotero:
         retrieved = self._retrieve_data(query_string)
         return self._cache(retrieved, template_name)
 
-    def _attachment_template(self, attachment_type):
+    def _attachment_template(self, attachment_type: str) -> Any:
         """Return a new attachment template of the required type.
 
         Types: imported_file, imported_url, linked_file, linked_url
         """
         return self.item_template("attachment", linkmode=attachment_type)
 
-    def _attachment(self, payload, parentid=None):
+    def _attachment(
+        self, payload: list[dict[str, Any]], parentid: str | None = None
+    ) -> Any:
         """Create attachments.
 
         Accepts a list of one or more attachment template dicts
@@ -824,17 +840,17 @@ class Zotero:
         return attachment.upload()
 
     @ss_wrap
-    def show_operators(self):
+    def show_operators(self) -> Any:
         """Show available saved search operators."""
         return self.savedsearch.operators
 
     @ss_wrap
-    def show_conditions(self):
+    def show_conditions(self) -> Any:
         """Show available saved search conditions."""
         return self.savedsearch.conditions_operators.keys()
 
     @ss_wrap
-    def show_condition_operators(self, condition):
+    def show_condition_operators(self, condition: str) -> set[Any]:
         """Show available operators for a given saved search condition."""
         # dict keys of allowed operators for the current condition
         permitted_operators = self.savedsearch.conditions_operators.get(condition)
@@ -842,7 +858,7 @@ class Zotero:
         return {self.savedsearch.operators.get(op) for op in permitted_operators}
 
     @ss_wrap
-    def saved_search(self, name, conditions):
+    def saved_search(self, name: str, conditions: list[dict[str, str]]) -> Any:
         """Create a saved search.
 
         conditions is a list of dicts containing search conditions and must
@@ -871,7 +887,7 @@ class Zotero:
         return req.json()
 
     @ss_wrap
-    def delete_saved_search(self, keys):
+    def delete_saved_search(self, keys: list[str]) -> int:
         """Delete one or more saved searches.
 
         Pass a list of one or more unique search keys.
@@ -896,11 +912,16 @@ class Zotero:
             self._set_backoff(backoff)
         return req.status_code
 
-    def upload_attachments(self, attachments, parentid=None, basedir=None):
+    def upload_attachments(
+        self,
+        attachments: Any,
+        parentid: str | None = None,
+        basedir: str | None = None,
+    ) -> Any:
         """Upload files to the already created (but never uploaded) attachments."""
         return Zupload(self, attachments, parentid, basedir=basedir).upload()
 
-    def add_tags(self, item, *tags):
+    def add_tags(self, item: dict[str, Any], *tags: str) -> bool:
         """Add one or more tags to a retrieved item, then update it on the server.
 
         Accepts a dict, and one or more tags to add to it.
@@ -915,13 +936,13 @@ class Zotero:
         self.check_items([item])
         return self.update_item(item)
 
-    def check_items(self, items):
+    def check_items(self, items: list[dict[str, Any]]) -> list[dict[str, Any]]:
         """Check that items to be created contain no invalid dict keys.
 
         Accepts a single argument: a list of one or more dicts.
         The retrieved fields are cached and re-used until a 304 call fails.
         """
-        params = {"locale": self.locale, "timeout": DEFAULT_TIMEOUT}
+        params: dict[str, Any] = {"locale": self.locale, "timeout": DEFAULT_TIMEOUT}
         query_string = "/itemFields"
         r = Request(
             "GET",
@@ -940,7 +961,7 @@ class Zotero:
         ):
             template = {t["field"] for t in self.templates[cachekey]["tmplt"]}
         else:
-            template = {t["field"] for t in self.item_fields()}
+            template = {t["field"] for t in self.item_fields()}  # ty: ignore[invalid-argument-type]
         # add fields we know to be OK
         template |= {
             "path",
@@ -975,7 +996,7 @@ class Zotero:
             "annotationAuthorName",
         }
         template |= set(self.temp_keys)
-        processed_items = []
+        processed_items: list[dict[str, Any]] = []
         for pos, item in enumerate(items):
             if set(item) == {"links", "library", "version", "meta", "key", "data"}:
                 itm = item["data"]
@@ -990,45 +1011,45 @@ class Zotero:
         return processed_items
 
     @tcache
-    def item_types(self):
+    def item_types(self) -> tuple[str, dict[str, Any]]:
         """Get all available item types."""
         # Check for a valid cached version
-        params = {"locale": self.locale}
+        params: dict[str, Any] = {"locale": self.locale}
         query_string = "/itemTypes"
         return query_string, params
 
     @tcache
-    def creator_fields(self):
+    def creator_fields(self) -> tuple[str, dict[str, Any]]:
         """Get localised creator fields."""
         # Check for a valid cached version
-        params = {"locale": self.locale}
+        params: dict[str, Any] = {"locale": self.locale}
         query_string = "/creatorFields"
         return query_string, params
 
     @tcache
-    def item_type_fields(self, itemtype):
+    def item_type_fields(self, itemtype: str) -> tuple[str, dict[str, Any]]:
         """Get all valid fields for an item."""
-        params = {"itemType": itemtype, "locale": self.locale}
+        params: dict[str, Any] = {"itemType": itemtype, "locale": self.locale}
         query_string = "/itemTypeFields"
         return query_string, params
 
     @tcache
-    def item_creator_types(self, itemtype):
+    def item_creator_types(self, itemtype: str) -> tuple[str, dict[str, Any]]:
         """Get all available creator types for an item."""
-        params = {"itemType": itemtype, "locale": self.locale}
+        params: dict[str, Any] = {"itemType": itemtype, "locale": self.locale}
         query_string = "/itemTypeCreatorTypes"
         return query_string, params
 
     @tcache
-    def item_fields(self):
+    def item_fields(self) -> tuple[str, dict[str, Any]]:
         """Get all available item fields."""
         # Check for a valid cached version
-        params = {"locale": self.locale}
+        params: dict[str, Any] = {"locale": self.locale}
         query_string = "/itemFields"
         return query_string, params
 
     @staticmethod
-    def item_attachment_link_modes():
+    def item_attachment_link_modes() -> list[str]:
         """Get all available link mode types.
 
         Note: No viable REST API route was found for this, so I tested and built
@@ -1037,7 +1058,12 @@ class Zotero:
         """
         return ["imported_file", "imported_url", "linked_file", "linked_url"]
 
-    def create_items(self, payload, parentid=None, last_modified=None):
+    def create_items(
+        self,
+        payload: list[dict[str, Any]],
+        parentid: str | None = None,
+        last_modified: int | None = None,
+    ) -> Any:
         """Create new Zotero items.
 
         Accepts two arguments:
@@ -1078,14 +1104,14 @@ class Zotero:
                 "If-Unmodified-Since-Version": req.headers["last-modified-version"],
             }
             for value in resp["success"].values():
-                payload = {"parentItem": parentid}
+                child_payload: dict[str, str] = {"parentItem": parentid}
                 self._check_backoff()
                 presp = self.client.patch(
                     url=build_url(
                         self.endpoint,
                         f"/{self.library_type}/{self.library_id}/items/{value}",
                     ),
-                    json=payload,
+                    json=child_payload,
                     headers=dict(uheaders),
                 )
                 self.request = presp
@@ -1098,11 +1124,19 @@ class Zotero:
                     self._set_backoff(backoff)
         return resp
 
-    def create_collection(self, payload, last_modified=None):
+    def create_collection(
+        self,
+        payload: list[dict[str, Any]],
+        last_modified: int | None = None,
+    ) -> Any:
         """Alias for create_collections to preserve backward compatibility."""
         return self.create_collections(payload, last_modified)
 
-    def create_collections(self, payload, last_modified=None):
+    def create_collections(
+        self,
+        payload: list[dict[str, Any]],
+        last_modified: int | None = None,
+    ) -> Any:
         """Create new Zotero collections.
 
         Accepts one argument, a list of dicts containing the following keys:
@@ -1140,7 +1174,9 @@ class Zotero:
         return req.json()
 
     @backoff_check
-    def update_collection(self, payload, last_modified=None):
+    def update_collection(
+        self, payload: dict[str, Any], last_modified: int | None = None
+    ) -> httpx.Response:
         """Update a Zotero collection property such as 'name'.
 
         Accepts one argument, a dict containing collection data retrieved
@@ -1161,7 +1197,7 @@ class Zotero:
             content=json.dumps(payload),
         )
 
-    def attachment_simple(self, files, parentid=None):
+    def attachment_simple(self, files: list[str], parentid: str | None = None) -> Any:
         """Add attachments using filenames as title.
 
         Args:
@@ -1178,7 +1214,11 @@ class Zotero:
             return self._attachment(to_add, parentid)
         return self._attachment(to_add)
 
-    def attachment_both(self, files, parentid=None):
+    def attachment_both(
+        self,
+        files: list[tuple[str, str]],
+        parentid: str | None = None,
+    ) -> Any:
         """Add child attachments using title, filename.
 
         Args:
@@ -1196,7 +1236,9 @@ class Zotero:
         return self._attachment(to_add)
 
     @backoff_check
-    def update_item(self, payload, last_modified=None):
+    def update_item(
+        self, payload: dict[str, Any], last_modified: int | None = None
+    ) -> httpx.Response:
         """Update an existing item.
 
         Accepts one argument, a dict containing Item data.
@@ -1214,7 +1256,7 @@ class Zotero:
             content=json.dumps(to_send),
         )
 
-    def update_items(self, payload):
+    def update_items(self, payload: list[dict[str, Any]]) -> bool:
         """Update existing items.
 
         Accepts one argument, a list of dicts containing Item data.
@@ -1240,7 +1282,7 @@ class Zotero:
                 self._set_backoff(backoff)
         return True
 
-    def update_collections(self, payload):
+    def update_collections(self, payload: list[dict[str, Any]]) -> bool:
         """Update existing collections.
 
         Accepts one argument, a list of dicts containing Collection data.
@@ -1267,7 +1309,9 @@ class Zotero:
         return True
 
     @backoff_check
-    def addto_collection(self, collection, payload):
+    def addto_collection(
+        self, collection: str, payload: dict[str, Any]
+    ) -> httpx.Response:
         """Add item to a collection.
 
         Accepts two arguments: The collection ID, and an item dict.
@@ -1287,7 +1331,9 @@ class Zotero:
         )
 
     @backoff_check
-    def deletefrom_collection(self, collection, payload):
+    def deletefrom_collection(
+        self, collection: str, payload: dict[str, Any]
+    ) -> httpx.Response:
         """Delete an item from a collection.
 
         Accepts two arguments: The collection ID, and an item dict.
@@ -1309,7 +1355,7 @@ class Zotero:
         )
 
     @backoff_check
-    def delete_tags(self, *payload):
+    def delete_tags(self, *payload: str) -> httpx.Response:
         """Delete a group of tags.
 
         Pass in up to 50 tags, or use *[tags].
@@ -1335,7 +1381,11 @@ class Zotero:
         )
 
     @backoff_check
-    def delete_item(self, payload, last_modified=None):
+    def delete_item(
+        self,
+        payload: dict[str, Any] | list[dict[str, Any]],
+        last_modified: int | None = None,
+    ) -> httpx.Response:
         """Delete Items from a Zotero library.
 
         Accepts a single argument:
@@ -1367,7 +1417,11 @@ class Zotero:
         return self.client.delete(url=url, params=params, headers=headers)
 
     @backoff_check
-    def delete_collection(self, payload, last_modified=None):
+    def delete_collection(
+        self,
+        payload: dict[str, Any] | list[dict[str, Any]],
+        last_modified: int | None = None,
+    ) -> httpx.Response:
         """Delete a Collection from a Zotero library.
 
         Accepts a single argument:
